/**
  ******************************************************************************
  * @file    stm32l0xx_it.c
  * @brief   Interrupt Service Routines.
  ******************************************************************************
  *
  * COPYRIGHT(c) 2017 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "stm32l0xx_hal.h"
#include "stm32l0xx.h"
#include "stm32l0xx_it.h"

/* USER CODE BEGIN 0 */
extern uint16_t tryb;
extern uint16_t repeat;
extern uint16_t tryb_next;
extern uint16_t relDelay;
extern uint16_t measDelay;
extern uint16_t blinkDelay;
extern uint16_t blink;
extern uint16_t timeout;
extern uint16_t piezo_cnt;
extern uint16_t piezoDelay;
extern int16_t temperature;
extern uint16_t repeat3;
extern uint16_t alarm;
extern uint16_t bat_meas;
extern uint16_t key1;
extern uint16_t key2;
extern uint16_t key3;
extern uint16_t key4;
extern uint16_t rel_turnoff;
/* USER CODE END 0 */

/* External variables --------------------------------------------------------*/
extern LPTIM_HandleTypeDef hlptim1;
extern TIM_HandleTypeDef htim6;

/******************************************************************************/
/*            Cortex-M0+ Processor Interruption and Exception Handlers         */ 
/******************************************************************************/

/**
* @brief This function handles Non maskable interrupt.
*/
void NMI_Handler(void)
{
  /* USER CODE BEGIN NonMaskableInt_IRQn 0 */

  /* USER CODE END NonMaskableInt_IRQn 0 */
  /* USER CODE BEGIN NonMaskableInt_IRQn 1 */

  /* USER CODE END NonMaskableInt_IRQn 1 */
}

/**
* @brief This function handles Hard fault interrupt.
*/
void HardFault_Handler(void)
{
  /* USER CODE BEGIN HardFault_IRQn 0 */

  /* USER CODE END HardFault_IRQn 0 */
  while (1)
  {
  }
  /* USER CODE BEGIN HardFault_IRQn 1 */

  /* USER CODE END HardFault_IRQn 1 */
}

/**
* @brief This function handles System service call via SWI instruction.
*/
void SVC_Handler(void)
{
  /* USER CODE BEGIN SVC_IRQn 0 */

  /* USER CODE END SVC_IRQn 0 */
  /* USER CODE BEGIN SVC_IRQn 1 */

  /* USER CODE END SVC_IRQn 1 */
}

/**
* @brief This function handles Pendable request for system service.
*/
void PendSV_Handler(void)
{
  /* USER CODE BEGIN PendSV_IRQn 0 */

  /* USER CODE END PendSV_IRQn 0 */
  /* USER CODE BEGIN PendSV_IRQn 1 */

  /* USER CODE END PendSV_IRQn 1 */
}

/**
* @brief This function handles System tick timer.
*/
void SysTick_Handler(void)
{
  /* USER CODE BEGIN SysTick_IRQn 0 */

  /* USER CODE END SysTick_IRQn 0 */
  HAL_IncTick();
  HAL_SYSTICK_IRQHandler();
  /* USER CODE BEGIN SysTick_IRQn 1 */

  /* USER CODE END SysTick_IRQn 1 */
}

/******************************************************************************/
/* STM32L0xx Peripheral Interrupt Handlers                                    */
/* Add here the Interrupt Handlers for the used peripherals.                  */
/* For the available peripheral interrupt handler names,                      */
/* please refer to the startup file (startup_stm32l0xx.s).                    */
/******************************************************************************/

/**
* @brief This function handles EXTI line 0 and line 1 interrupts.
*/
void EXTI0_1_IRQHandler(void)
{
  /* USER CODE BEGIN EXTI0_1_IRQn 0 */
	if ((tryb == 7) & (alarm == 0)) { //BTN4
		tryb_next = 1;
	} else if ((tryb == 7) & (alarm > 5)) {
		tryb_next = 8;
	}
  /* USER CODE END EXTI0_1_IRQn 0 */
  HAL_GPIO_EXTI_IRQHandler(GPIO_PIN_0);
  /* USER CODE BEGIN EXTI0_1_IRQn 1 */

  /* USER CODE END EXTI0_1_IRQn 1 */
}

/**
* @brief This function handles EXTI line 2 and line 3 interrupts.
*/
void EXTI2_3_IRQHandler(void)
{
  /* USER CODE BEGIN EXTI2_3_IRQn 0 */
	if ((tryb == 7) & (alarm > 5)) {
			tryb_next = 8;
		} else {
			tryb_next = 1;
		}
  /* USER CODE END EXTI2_3_IRQn 0 */
  HAL_GPIO_EXTI_IRQHandler(GPIO_PIN_2);
  /* USER CODE BEGIN EXTI2_3_IRQn 1 */

  /* USER CODE END EXTI2_3_IRQn 1 */
}

/**
* @brief This function handles EXTI line 4 to 15 interrupts.
*/
void EXTI4_15_IRQHandler(void)
{
  /* USER CODE BEGIN EXTI4_15_IRQn 0 */
	if (__HAL_GPIO_EXTI_GET_IT(GPIO_PIN_14) != RESET) {
		//Wlaczono zasilanie sieciowe
		tryb_next = 1; //bylo 3
		if ((alarm == 6) | (alarm == 7)) {
			alarm = 0;
			piezo_cnt = 0;
			bat_meas = 2;
		}
	} else if (__HAL_GPIO_EXTI_GET_IT(GPIO_PIN_5) != RESET) {
		if ((tryb == 7) & (alarm > 5)) {
			tryb_next = 8;
		} else {
			tryb_next = 1;
		}
	} else if (__HAL_GPIO_EXTI_GET_IT(GPIO_PIN_4) != RESET) {
		if ((tryb == 7) & (alarm > 5)) {
			tryb_next = 8;
		} else {
			tryb_next = 1;
		}
	}
  /* USER CODE END EXTI4_15_IRQn 0 */
  HAL_GPIO_EXTI_IRQHandler(GPIO_PIN_4);
  HAL_GPIO_EXTI_IRQHandler(GPIO_PIN_5);
  HAL_GPIO_EXTI_IRQHandler(GPIO_PIN_14);
  /* USER CODE BEGIN EXTI4_15_IRQn 1 */

  /* USER CODE END EXTI4_15_IRQn 1 */
}

/**
* @brief This function handles LPTIM1 global interrupt / LPTIM1 wake-up interrupt through EXTI line 29.
*/
void LPTIM1_IRQHandler(void)
{
  /* USER CODE BEGIN LPTIM1_IRQn 0 */
	if (measDelay == 0) {
		measDelay = 1;
	}
	if ((tryb == 7) & (alarm < 5)){
		tryb_next = 2; //Przy wybudzeniu automatycznym
	}
  /* USER CODE END LPTIM1_IRQn 0 */
  HAL_LPTIM_IRQHandler(&hlptim1);
  /* USER CODE BEGIN LPTIM1_IRQn 1 */

  /* USER CODE END LPTIM1_IRQn 1 */
}

/**
* @brief This function handles TIM6 global interrupt and DAC1/DAC2 underrun error interrupts.
*/
void TIM6_DAC_IRQHandler(void)
{
  /* USER CODE BEGIN TIM6_DAC_IRQn 0 */
	//Obsluga przyciskow
	if ((key1 > 0) & (key1 < 3)) {
		key1++;
	} else if ((key1 > 3) & (key1 < 21)) {
		key1++;
	} else if ((key1 > 21) & (key1 < 90)) {
		key1++;
		if (key1 > 27) {
			key1 = 21;
		}
	}
	if ((key2 > 0) & (key2 < 3)) {
		key2++;
	} else if ((key2 > 3) & (key2 < 21)) {
		key2++;
	} else if ((key2 > 21) & (key2 < 90)) {
		key2++;
		if (key2 > 27) {
			key2 = 21;
		}
	}
	if ((key3 > 0) & (key3 < 3)) {
		key3++;
	} else if ((key3 > 3) & (key3 < 21)) {
		key3++;
	} else if ((key3 > 21) & (key3 < 90)) {
		key3++;
		if (key3 > 27) {
			key3 = 21;
		}
	}
	if ((key4 > 0) & (key4 < 3)) {
		key4++;
	} else if ((key4 > 3) & (key4 < 31)) {
		key4++;
	}
	//Obsluga przekaznika
	if (relDelay > 0) {
		relDelay--;
		if (relDelay == 0) {
			GPIOA->BSRR = GPIO_PIN_11 << 16;
			GPIOA->BSRR = GPIO_PIN_12 << 16;
		} else if (relDelay == 1) { //kontrola stanu baterii
			if (bat_meas == 0) {
				bat_meas = 1;
			}
		}
	}
	//Obsluga opoznienia pomiaru temperatury
	if (measDelay > 2) {
		measDelay--;
	}
	
	//Obsluga opoznienia zmiany trybow
	if (timeout > 0) {
		timeout--;
		if (timeout == 0) {
			if (tryb == 0) {
				if (bat_meas == 3) { //Zaleznie od stanu baterii
					alarm = 7;
					tryb_next = 5;
				} else {
					tryb_next = 1;
				}
			} else if (tryb == 1) {
				if (temperature < 201) {
					if ((alarm == 1) | (alarm == 3)) {
						repeat3 = 1;
					} else {
						tryb_next = 7;
					}
				} else {
					tryb_next = 8;
				}
			} else if (tryb == 3) {
				tryb_next = 1;
			} else if (tryb == 4) {
				if (repeat == 0) {
					repeat = 1;
				}
			}
		}
	}
	
	//Obsluga migania LED
	if (blinkDelay > 0) {
		blinkDelay--;
		if (blinkDelay == 0) {
			blinkDelay = 7;
			//Miganie
			blink++;
			blink &= 0x0001;
		}
	}
	
	//Obsluga piezo
	if (piezoDelay > 0) {
		piezoDelay--;
		if (piezoDelay == 0) {
			if (piezo_cnt > 0) {
				piezoDelay = 7;
				piezo_cnt--;
				if (piezo_cnt & 0x0001) {
					//Wylacz piezo
					GPIOC->BSRR = GPIO_PIN_13 << 16;
				} else {
					GPIOC->BSRR = GPIO_PIN_13;
				}
			} else {
				GPIOC->BSRR = GPIO_PIN_13 << 16;
				rel_turnoff = 1;
				if ((repeat == 2) & (tryb == 4)) {
					repeat = 3;
				}
				if (tryb == 5) {
					bat_meas = 4;
					alarm = 7;
					tryb_next = 6;
				}
			}
		}
	}	
  /* USER CODE END TIM6_DAC_IRQn 0 */
  HAL_TIM_IRQHandler(&htim6);
  /* USER CODE BEGIN TIM6_DAC_IRQn 1 */

  /* USER CODE END TIM6_DAC_IRQn 1 */
}

/* USER CODE BEGIN 1 */

/* USER CODE END 1 */
/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
